@@ -1,5 +1,5 @@
 from dataclasses import dataclass
-from typing import Generic, Tuple, Type
+from typing import Generic, Type
 
 import torch
 import torch.nn as nn
@@ -13,13 +13,11 @@ from mohou.types import ImageT, ImageBase
 class AutoEncoderConfig(ModelConfigBase):
     image_type: Type[ImageBase]
     n_bottleneck: int = 16
-    input_shape: Tuple[int, int] = (224, 224)
+    n_pixel: int = 112
 
     def __post_init__(self):
         # validation
-        n_pixel, m_pixel = self.input_shape
-        assert n_pixel == m_pixel
-        assert n_pixel in [28, 112, 224]
+        assert self.n_pixel in [28, 112, 224]
 
 
 class Reshape(nn.Module):
@@ -36,6 +34,7 @@ class AutoEncoder(ModelBase[AutoEncoderConfig], Generic[ImageT]):
     image_type: Type[ImageT]
     encoder: nn.Module
     decoder: nn.Module
+    n_pixel: int
 
     def loss(self, sample: torch.Tensor) -> LossDict:
         f_loss = nn.MSELoss()
@@ -45,12 +44,12 @@ class AutoEncoder(ModelBase[AutoEncoderConfig], Generic[ImageT]):
 
     def forward(self, input: torch.Tensor) -> torch.Tensor:
         assert input.ndim == 4
+        assert list(input.shape[2:]) == [self.n_pixel, self.n_pixel]
         assert self.image_type.channel() == input.shape[1], 'channel mismatch'
         return self.decoder(self.encoder(input))
 
     def get_embedder(self) -> ImageEmbedder[ImageT]:
-        shape = self.config.input_shape
-        np_image_shape = (shape[0], shape[1], self.channel())
+        np_image_shape = (self.config.n_pixel, self.config.n_pixel, self.channel())
         embedder = ImageEmbedder[ImageT](
             self.image_type,
             lambda image_tensor: self.encoder(image_tensor),
@@ -64,7 +63,8 @@ class AutoEncoder(ModelBase[AutoEncoderConfig], Generic[ImageT]):
 
     def _setup_from_config(self, config: AutoEncoderConfig):
         self.image_type = config.image_type  # type: ignore
-        n_pixel, m_pixel = config.input_shape
+        n_pixel = config.n_pixel
+        self.n_pixel = n_pixel
 
         # TODO(HiroIshida) do it programatically
         if n_pixel == 224:
